@@ -21,6 +21,7 @@ import unittest, os, shutil, time
 from roundup.backends import get_backend, have_backend
 
 from db_test_base import DBTest, ROTest, SchemaTest, ClassicInitTest, config
+from db_test_base import ConcurrentDBTest, FilterCacheTest
 
 class sqliteOpener:
     if have_backend('sqlite'):
@@ -41,6 +42,12 @@ class sqliteSchemaTest(sqliteOpener, SchemaTest):
 class sqliteClassicInitTest(ClassicInitTest):
     backend = 'sqlite'
 
+class sqliteConcurrencyTest(ConcurrentDBTest):
+    backend = 'sqlite'
+
+class sqliteFilterCacheTest(sqliteOpener, FilterCacheTest):
+    backend = 'sqlite'
+
 from session_common import RDBMSTest
 class sqliteSessionTest(sqliteOpener, RDBMSTest):
     pass
@@ -57,6 +64,8 @@ def test_suite():
     suite.addTest(unittest.makeSuite(sqliteSchemaTest))
     suite.addTest(unittest.makeSuite(sqliteClassicInitTest))
     suite.addTest(unittest.makeSuite(sqliteSessionTest))
+    suite.addTest(unittest.makeSuite(sqliteConcurrencyTest))
+    suite.addTest(unittest.makeSuite(sqliteFilterCacheTest))
     return suite
 
 if __name__ == '__main__':
